/***************************************************************************
 *   Copyright (C) 2001 AVM GmbH. All rights reserved.                     *
 *   Copyright (C) 1998-2000 Markus Dahlweid                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kimoninfoline.h"
#include "kimoninfoline.moc"

#include <stdio.h>

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
KIsdnInfoLine::KIsdnInfoLine (QWidget* parent, bool panel, const char* name)
						 : QWidget(parent, name)
{
    _upsince = QDateTime::currentDateTime ();
    
    _panel = panel;
    _onl = false;
    
    m_pLabelController = new QLabel (this);
    m_pLabelController->setFixedSize (60, 20);
    if ( _panel ) m_pLabelController->setFrameStyle (QFrame::Panel|QFrame::Sunken);
    m_pLabelController->setAlignment (AlignHCenter | AlignVCenter);
    m_pLabelController->setText ("");
    m_pLabelController->move (10,3);
    m_pLabelController->show ();

    m_pLabelChannel = new QLabel (this);
    m_pLabelChannel->setFixedSize (60,20);
    if ( _panel ) m_pLabelChannel->setFrameStyle (QFrame::Panel|QFrame::Sunken);
    m_pLabelChannel->setAlignment (AlignHCenter | AlignVCenter);
    m_pLabelChannel->setText ("");
    m_pLabelChannel->move (72,3);
    m_pLabelChannel->show ();
    
    m_pLabelStatus = new QLabel (this);
    m_pLabelStatus->setFixedSize(60,20);
    if ( _panel ) m_pLabelStatus->setFrameStyle (QFrame::Panel|QFrame::Sunken);
    m_pLabelStatus->setAlignment (AlignHCenter|AlignVCenter);
    m_pLabelStatus->setText ("Offline");
    m_pLabelStatus->move (134,3);
    m_pLabelStatus->show ();
    
    m_pLabelTime = new QLabel (this);
    m_pLabelTime->setFixedSize (60,20);
    if ( _panel ) m_pLabelTime->setFrameStyle( QFrame::Panel|QFrame::Sunken);
    m_pLabelTime->setAlignment (AlignHCenter|AlignVCenter);
    m_pLabelTime->setText ("0:00:00");
    m_pLabelTime->move (196,3);
    m_pLabelTime->show ();

    if ( _panel )
    {
      	// This is necessary because of a bug in kled.h
      	QWidget *_temp = new QWidget (this);
      	_temp->setGeometry (258,6, 14,14);
      	
      	m_pLed = new KIsdnInfoLineStatus (_temp);
      	m_pLed->setFixedSize (14,14);
      	m_pLed->show ();
      	m_pLed->on ();
      	_temp->show ();
    }
    else
    {
    		m_pLabelController->setText ("Controller");
    		m_pLabelChannel->setText ("B Channel");
    		m_pLabelStatus->setText ("Status");
    		m_pLabelTime->setText ("Time");
    		
				m_pLed = NULL;
    }
    
    setFixedSize (279,23);
}

KIsdnInfoLine::~KIsdnInfoLine()
{
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_controller (int iController)
{
    m_pLabelController->setNum (iController);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_controller (const char* controller)
{
    m_pLabelController->setText (controller);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_channel (int iChannel)
{
    m_pLabelChannel->setNum (iChannel);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_channel (const char* channel)
{
    m_pLabelChannel->setText(channel);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_online (int dir)
{
    if ( dir != _onl )
    {
        if ( dir == 0 )
        {
	    			m_pLed->setColor(KLed::darkGray);
    				m_pLabelStatus->setText ("Offline");
	    	}
        else
        {
        		reset_time ();
           	m_pLed->setColor(KLed::green);
    				m_pLabelStatus->setText ("Online");
        }
				_onl = dir;
    }
    recalc_time ();
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::set_time (const char *time)
{
    m_pLabelTime->setText (time);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::reset_time ()
{
    _upsince = QDateTime::currentDateTime ();
    sprintf (timestr, "0:00:00");
    set_time (timestr);
}

/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
void KIsdnInfoLine::recalc_time ()
{
    if ( _onl != 0 )
    {
				dur = _upsince.secsTo (QDateTime::currentDateTime ());
				sec = dur % 60;
				min = dur / 60;
				hou = min / 60;
				min = min % 60;
				sprintf (timestr, "%d:%02d:%02d", hou, min, sec);
    }
    else sprintf (timestr, "0:00:00");

    set_time (timestr);
}
